// ADT7420 I2C temperature sensor controller for Nexys4

module adt7420_read
  (
   input 	 CLK, RST,
   input 	 GO,
   output 	 BUSY,
   output [15:0] DATA,
   output 	 STROBE,
   output 	 SCL,
   inout 	 SDA
   );

   //   parameter ClockCount = 1;
   parameter ClockCount = 500; // period = 500: 200kHz at 100MHz clock
   reg [9:0] 	CLK_CNT;
   
   
   reg [4:0] STAT;  // IDLE, START, ADR, DAT1, DAT2
   reg [8:0] BIT;

   wire      STAT_IDLE = STAT[4];
   wire      STAT_START = STAT[3];
   wire      STAT_ADR = STAT[2];
   wire      STAT_DAT1 = STAT[1];
   wire      STAT_DAT2 = STAT[0];
   
  
   // --------------------------------------------------
   // Clock generation
   
   reg       SCL_INT;
   reg 	     SCL_R;
   wire      SCL_NEGEDGE = SCL_R & ~SCL_INT;
   wire      SCL_POSEDGE = ~SCL_R & SCL_INT;

   always @ (posedge CLK) begin
      SCL_R <= SCL_INT;
      
      if (RST) begin
	 SCL_INT <= 1'b1;
      end else begin
	 if (!STAT_IDLE) begin
	    CLK_CNT <= (CLK_CNT == ClockCount) ? 0 : CLK_CNT+1;
	    if (CLK_CNT == 0) SCL_INT <= ~SCL_INT;
	 end else begin
	    SCL_INT <= 1;
	    CLK_CNT <= 0;
	 end
      end
   end


   reg SCL_MASK;
   always @ (posedge CLK) begin
      if (BIT[1] & STAT_START) SCL_MASK <= 1;
      else if (SCL_INT) SCL_MASK <= 0;
   end
   
//   assign SCL = STAT_START ? 0 : SCL_INT;
   assign SCL = SCL_MASK | SCL_INT;
   

   // --------------------------------------------------
   // SDA control

   wire  SDA_O;
   wire  SDA_DRIVE;

   assign SDA = SDA_DRIVE ? SDA_O : 1'bz;
   assign SDA_DRIVE = STAT_IDLE | STAT_START | ( STAT_ADR & ~BIT[0] ) | 
		      ( (STAT_DAT1 | STAT_DAT2) & BIT[0]);
   
   assign SDA_O = ( STAT_IDLE ? 1 :
		    STAT_START ? 0 :
		    STAT_ADR ? |(8'b1001_0111 & BIT[8:1]) :
		    STAT_DAT1 ? 0 : // Ack
		    STAT_DAT2 ? 1 : // No ack
		    1'hx );
   

   // --------------------------------------------------
   // Main state machine
   
   always @ (posedge CLK) begin
      if (RST) begin
	 STAT <= 5'b1_0000;
	 BIT  <= 9'b1_0000_0000;
	
      end else begin

	 if (|STAT[3:0]) // START, ADR, DAT1, DAT2
	   if (SCL_NEGEDGE) BIT <= {BIT[0], BIT[8:1]};
	 
	 casex (STAT)
	   5'b1_0000: begin // IDLE
	      if (GO) begin
		 STAT <= {STAT[0], STAT[4:1]};
		 BIT  <= 9'b0_0000_0010;
		 end
	   end

	   5'b0_xxxx: begin // START, ADR, DAT1, DAT2
	      if (SCL_NEGEDGE & BIT[0] )  STAT <= {STAT[0], STAT[4:1]};
	   end
	 endcase // case (STAT)
	 
      end
   end
   

   // --------------------------------------------------
   // Receiver

   reg [15:0] DAT_REG;
   
   always @ (posedge CLK) begin
      if (GO) DAT_REG <= 16'b0;
      else begin
	 if (SCL_POSEDGE) begin
	    case (STAT[1:0])
	      2'b10: DAT_REG[15:8] <= DAT_REG[15:8] | (SDA ? BIT[8:1]:8'b0);
	      2'b01: DAT_REG[ 7:0] <= DAT_REG[ 7:0] | (SDA ? BIT[8:1]:8'b0);
	    endcase
	 end
      end
   end

   assign DATA = DAT_REG;
   assign STROBE = STAT_DAT2 & BIT[0] & SCL_POSEDGE;

   // --------------------------------------------------
   // Misc
   
   assign BUSY = GO | ~STAT_IDLE;

endmodule // adt7420_read


