/*
   Nexys4 demo kit
     
   7 segment display: 
      SW[0] : Temperature
      SW[1] : Accelerometer values
      SW[2] : PS2 input value
      SW[3] : PS2
      SW[4] : PS2 input value (mouse mode)
      SW[5] : Microphone double / single clap counter
 
   Tri-color LEDs:
      SW[14] : Mouse
      SW[15] : Accelerometer
 
   Discrete green LEDs:
      SW[13] : PWM demo
      SW[12]+SW[11] : Microphone power meter
 */


module top
  ( input CLK,
    input         RST, // center button
     
    output        ADXL_CS_N,
    output        ADXL_SCLK, 
    output        ADXL_MOSI,
    input         ADXL_MISO,

    output        TMP_SCL, 
    inout         TMP_SDA,

    output        ADMP_CLK, ADMP_LR,
    input         ADMP_DATA,

    input         PS2_CLK, PS2_DAT,
  
    output [2:0]  TRI_LED1, TRI_LED2,

    output [15:0] LED,

    input [15:0]  SW,
    input         BTN_L,
    
    output [7:0]  ANODE,
    output [7:0]  CATHODE );


   // 27bits: 1Hz, 24bits: 8Hz
   reg [23:0] 	  CNT;
   always @ (posedge CLK) 
     CNT <= RST ? 0 : CNT + 1;

   wire [31:0] 	  DISP_PS2_MOUSE, DISP_PS2_CNT, DISP_PS2,
		  DISP_ADXL, DISP_ADT, DISP_VAL, DISP_MIC;

   wire [2:0] 	  ACCEL_LED, MOUSE_LED;

   wire [15:0]    PWM_LED, PMETER_LED;

   // --------------------------------------------------
   // 7 Segment Display
   
   assign DISP_VAL = SW[0] ? DISP_ADT :
		     SW[1] ? DISP_ADXL :
		     SW[2] ? DISP_PS2 :
		     SW[3] ? DISP_PS2_CNT :
		     SW[4] ? DISP_PS2_MOUSE :
		     SW[5] ? DISP_MIC :
		     0;
   
   segment_disp_ctrl sdc ( .CLK(CLK), .RST(RST), .VAL(DISP_VAL),
			   .ANODE(ANODE), .CATHODE(CATHODE) );
   

   // --------------------------------------------------
   // Tri-Color LED
   
   assign TRI_LED1 = SW[15] ? ACCEL_LED : 0;
   assign TRI_LED2 = SW[14] ? MOUSE_LED : 0;

   // --------------------------------------------------
   // Discrete LED

   assign LED = SW[13] ? PWM_LED :
                SW[12] ? PMETER_LED :
                0;
   
   // SW[11] for powermeter source selection
   
   // --------------------------------------------------
   // PS/2 Mouse

   wire [15:0] 	  MOUSE_X, MOUSE_Y;
   wire 	  MOUSE_STROBE, MOUSE_L, MOUSE_R;
   reg [31:0] 	  MOUSE_POS;
   
   ps2mouse p2m ( .CLK(CLK), .RST(RST), .PS2_CLK(PS2_CLK), .PS2_DAT(PS2_DAT),
		  .STROBE(MOUSE_STROBE), .X(MOUSE_X), .Y(MOUSE_Y),
		  .LEFT(MOUSE_L), .RIGHT(MOUSE_R) );
   

   always @ (posedge CLK)
     MOUSE_POS <= { MOUSE_X, MOUSE_Y };
   
   assign DISP_PS2_MOUSE = MOUSE_POS;

   wire 	  MOUSE_LED_TEMP;
   duty_ctrl mb_dc ( .CLK(CLK), .DUTY(64), .Q(MOUSE_LED_TEMP) );

   assign MOUSE_LED = { (MOUSE_L ? MOUSE_LED_TEMP : 1'b0 ),
			(MOUSE_R ? MOUSE_LED_TEMP : 1'b0 ), 1'b0 };
 
      
   // --------------------------------------------------
   // PS/2

   wire 	  PS2_STROBE;
   wire [7:0] 	  PS2_CODE;

   ps2recv p2r ( .CLK(CLK), .RST(RST), .PS2_CLK(PS2_CLK), .PS2_DAT(PS2_DAT),
		 .STROBE(PS2_STROBE), .KEYCODE(PS2_CODE) );
   
   reg [31:0] 	  PS2_CODES, PS2_CNT;
   
   always @ (posedge CLK) begin
      if (RST) begin
	PS2_CODES <= 0;
	 PS2_CNT <= 0;
      end else begin
	 if (PS2_STROBE) begin
	   PS2_CODES <= {PS2_CODES[23:0], PS2_CODE};
	    PS2_CNT <= PS2_CNT + 1;
	 end
      end
   end // always @ (posedge CLK)
   
   assign DISP_PS2 = PS2_CODES;
   assign DISP_PS2_CNT = PS2_CNT;
      
   
   // --------------------------------------------------
   // Accelerometer
   
   wire [7:0] 	  ADXL_X, ADXL_Y, ADXL_Z;
   
   wire 	  ADXL_STROBE;
   wire 	  ADXL_GO = &CNT;

   adxl362_read adxl ( .CLK(CLK), .RST(RST), .GO(ADXL_GO),
		       .X(ADXL_X), .Y(ADXL_Y), .Z(ADXL_Z),
		       .SCLK(ADXL_SCLK), .CS_N(ADXL_CS_N),
		       .MOSI(ADXL_MOSI), .MISO(ADXL_MISO),
		       .STROBE(ADXL_STROBE) );

   reg [23:0] 	  ACCEL;

   always @ (posedge CLK)
     if (ADXL_STROBE) ACCEL <= {ADXL_X, ADXL_Y, ADXL_Z};

   assign DISP_ADXL = {8'b0, ACCEL};
   
   // --------------------------------------------------
   // Temperature Sensor
   
   wire [15:0] 	  ADT_OUT;
   wire 	  ADT_STROBE;
   wire 	  ADT_GO = &CNT;
   
   reg [15:0] 	  TEMP;
   
   adt7420_read adt ( .CLK(CLK), .RST(RST), .GO(ADT_GO),
		      .DATA(ADT_OUT), .STROBE(ADT_STROBE),
		      .SCL(TMP_SCL), .SDA(TMP_SDA) );

   always @ (posedge CLK) 
     if (ADT_STROBE) TEMP <= ADT_OUT;

   assign DISP_ADT = {16'b0, TEMP};

   // --------------------------------------------------
   // Accelerometer to Tri-color LED, but not full brightness

   duty_ctrl acd1 ( .CLK(CLK), .DUTY({2'b0, ACCEL[23:18]}), .Q(ACCEL_LED[2]) );
   duty_ctrl acd2 ( .CLK(CLK), .DUTY({2'b0, ACCEL[15:10]}), .Q(ACCEL_LED[1]) );
   duty_ctrl acd3 ( .CLK(CLK), .DUTY({2'b0, ACCEL[ 7: 2]}), .Q(ACCEL_LED[0]) );

   // --------------------------------------------------
   // LED kurukuru with PWM

   reg [7:0] 	  DUTY[15:0];
   

   always @ (posedge CLK)  if (&CNT) DUTY[15] <= DUTY[ 0];

   genvar i;
   generate
      for (i=0; i<15; i=i+1) begin : duty_shift
         always @ (posedge CLK)  if (&CNT) DUTY[i] <= DUTY[i+1];
      end
   endgenerate

   generate
      for (i=0; i<16; i=i+1) begin : duty_gen
         initial DUTY[i] <= (i+1) * 15; // DUTY[0]=15, DUTY[15]=240
         
         duty_ctrl dc_i ( .CLK(CLK), .DUTY(DUTY[i]), .Q(PWM_LED[i]) );
      end
   endgenerate
   

   // --------------------------------------------------
   // Power meter to LED

   wire [6:0] MIC_Q;
   wire       MIC_VALID;

   wire [7:0] PMETER_D = SW[11] ? {MIC_Q, 1'b0} : (BTN_L ? 8'hff : 0);
   wire       PMETER_D_STROBE = SW[11] ? MIC_VALID : BTN_L;
   
   powermeter pmeter
     (
      .CLK(CLK), .RST(RST),
      .D(PMETER_D),
      .D_STROBE(PMETER_D_STROBE),
      .LED(PMETER_LED)
      );
   

   admp421_read mic
     (
      .CLK(CLK), .RST(RST),
      .ADMP421_CLK(ADMP_CLK),
      .ADMP421_LR(ADMP_LR),
      .ADMP421_DATA(ADMP_DATA),
      .Q(MIC_Q),
      .STROBE(MIC_VALID)
      );

   // MIC peak detect & 0.1sec mask
   
   reg [26:0] MIC_MASK_CNT; // MIC trigger mask for 0.1sec
   wire       MIC_PEAK = ( MIC_VALID & 
                           // (MIC_Q[6:3] == 4'hf | MIC_Q[6:3] == 4'he) &
                           // (&MIC_Q[6:5]) &  // c, d, e, f
                           (&MIC_Q[6:5] | &{MIC_Q[6], MIC_Q[4]}) & // a, b, c, d, e, f
                           (MIC_MASK_CNT == 0) );
   
   always @ (posedge CLK) 
      MIC_MASK_CNT <= RST ? 0 :
                      MIC_PEAK ? ( 10 * 1000 * 1000 ) :
                      ( MIC_MASK_CNT == 0 ? 0 : MIC_MASK_CNT-1 );

   // clap counter

   reg [7:0]  CLAP_CNT;
   always @ (posedge CLK)
     CLAP_CNT <= RST ? 0 :
                 MIC_PEAK ? CLAP_CNT + 1 :
                 CLAP_CNT;

   // double clap counter

   reg [26:0] DOUBLE_CLAP_TIMER; // double clap timer for 0.5sec
   wire       DOUBLE_CLAP_RUNNING = (DOUBLE_CLAP_TIMER != 0);
   wire       DOUBLE_CLAP = DOUBLE_CLAP_RUNNING & MIC_PEAK;
   
   always @ (posedge CLK) 
     DOUBLE_CLAP_TIMER <= RST ? 0 :
                          ( MIC_PEAK & ~DOUBLE_CLAP_RUNNING) ? ( 50 * 1000 * 1000 ) :
                          ( MIC_PEAK &  DOUBLE_CLAP_RUNNING) ? 0 :
                          (~MIC_PEAK &  DOUBLE_CLAP_RUNNING) ? DOUBLE_CLAP_TIMER - 1 :
                          0;

   reg [7:0]  DOUBLE_CLAP_CNT;
   
   always @ (posedge CLK)
     DOUBLE_CLAP_CNT <= RST ? 0 :
                        DOUBLE_CLAP ? DOUBLE_CLAP_CNT + 1 :
                        DOUBLE_CLAP_CNT;
   
   // output
   
   assign DISP_MIC = {16'h0, DOUBLE_CLAP_CNT, CLAP_CNT};


   
endmodule
